/*=============================================================================

                            POMME
                            -----
 
  pomme_idl.c is a IDL 'call_external' interface calling the 'POMME' version
  2.x and 3 code from Stefan Maus. See 'call_external' documentation in RSI IDL
  manual.

     -> pomme_init         
     -> pomme_calc            
     -> pomme_control

  2004-Nov-01. Martin Rother (rother@gfz-potsdam.de).
  2006-Jan-26. just some naming changes on moving to pomme-version 3.x.
=============================================================================*/

#include <stdio.h>

#include <pomme3d1.h>
#include <idl_export.h>

long pomme_init_slave
(
   IDL_STRING  *pomme_cf,
   IDL_STRING  *sm2geo_cf,
   IDL_STRING  *gsm2geo_cf,
   int         *verbose
)
{
   /*--------------------------------------------------------------------------

   Slave routine by interfacing 'pomme_init'.

   2004-Mar-29. Martin Rother (rother@gfz-potsdam.de).
   2006-Jan-26. just some naming changes on moving to pomme-version 3.x.
   -------------------------------------------------------------------------*/
   int    status;
   tmodel *model;
   tmodel *remodel;
   long   lmodel;

   model = (tmodel*) calloc((size_t)1, (size_t)sizeof(tmodel));
   if (!model)
   {
      printf("model could not created\n");
      return(NULL);
   }

   status = pomme_open_file_mp
      (
         pomme_cf->s,
         sm2geo_cf->s,
         gsm2geo_cf->s,
         model,
         sizeof(tmodel),
         *verbose
      );
   if (status)
   {
      printf("illegal status!\n");
      free(model);              /* something wrong */
      return(NULL);
   }

   lmodel = (long)model;
   return(lmodel);
}

void pomme_kill
(
   int argc,
   void *argv[]
)
{
   /* uh, that may be desastrous... */

   long   *lmodel;
   tmodel *model;

   lmodel = (long*)   argv[0];
   model  = (tmodel*) *lmodel;

   if (model)
   {
      free(model);
   }
   return;
}

long pomme_init
(
   int   argc, 
   void  *argv[]
)
{
   /*-------------------------------------------------------------------------

   Master routine interfacing to calling IDL
   (with IDL portable calling convension)

   input:

      argv[0]    string       pomme coefficient path/file
      argv[1]    string       transformation parameter file path
      argv[2]    integer      debug flag (should be off, if no special reason

   returns:

      uh, something.

                 long         noting directly usable...

   -------------------------------------------------------------------------*/
   long   lmodel;
   
   if (argc != 4)
   { 
      return ( (long)0 );       /* unexpected */
   }
   lmodel = pomme_init_slave
      (
         (IDL_STRING *) argv[0], /* pomme coefficient file path */
         (IDL_STRING *) argv[1], /* SM parameter file path */
         (IDL_STRING *) argv[2], /* GSM parameter file path */
         (int*)         argv[3]  /* verbose on/off flag */
      );
   return(lmodel);               /* a long int containing a pointer... */
}
   
/*==========================================================================*/

int pomme_calc_slave
(
   long   *lmodel,
   long   *len,
   double *fday, 
   double *rad,
   double *theta,
   double *phi,
   double *est,
   double *ist,
   double *imf_by,
   double *bm,
   double *be
)
{
   /*-------------------------------------------------------------------------

   Slave routine interfacing to calling IDL
   (with IDL portable calling convension, very conservative!)

   input:

   returns:

      uh, something.

                 int          status, not zero, if something was wrong..

   -------------------------------------------------------------------------*/
   long     stati, state;
   double   xm, ym, zm, fm;
   double   xe, ye, ze;
   tmodel   *model;
   long     i, l;
   double   theta_deg, phi_deg;

   /* CHECK LENGTH PAR */

   l = *len;
   if (l < 0)                   /* no idea... */
   {
      return(0);
   }
   model = (tmodel*)*lmodel;    /* this should be exactly this way */

   for( i = 0; i < l; i++)
   {
      theta_deg = theta[i] * M_PI/(double)180.0; /* required in rad */
      phi_deg   = phi[i]   * M_PI/(double)180.0; /* required in rad */

      stati = pomme_internal_field_mp
         (
            model,              /* input: pomme coefficient file path */
            fday[i],            /* input: fday */
            rad[i],             /* input: rad  */
            theta_deg,          /* input: theta in radians */          
            phi_deg,            /* input: phi in radians */
            &xm,                /* output: internal X */
            &ym,                /* output: internal Y */
            &zm,                /* output: internal Z */
            &fm                 /* output: internal F */
         );
      state = pomme_external_field_mp
         (
            model,              /* input: pomme coefficient file path */
            fday[i],            /* input: fday */
            rad[i],             /* input: rad  */
            theta_deg,          /* input: theta in radians */          
            phi_deg,            /* input: phi in radians */
            est[i],             /* input: est    parameter */
            ist[i],             /* input: ist    parameter */
            imf_by[i],          /* input: imf_by parameter */
            &xe,                /* output: external X */
            &ye,                /* output: external Y */
            &ze                 /* output: external Z */
         );         
      bm[4*i    ] = xm;         /* no checks here! */
      bm[4*i + 1] = ym;
      bm[4*i + 2] = zm;
      bm[4*i + 3] = fm;
      be[3*i    ] = xe;         /* no checks here! */
      be[3*i + 1] = ye;
      be[3*i + 2] = ze;
   }

   return(stati + 100*state);   /* probably unused */
}

int pomme_calc
(
   int   argc, 
   void  *argv[]
)
{
   /*-------------------------------------------------------------------------

   Master routine interfacing to calling IDL
   (with IDL portable calling convension)

   input:

      argv[0]    long         pointer to structure created by 'pomme_init'
      argv[1]    long         number of elements in following parameters
      argv[2]    double*      date/time word (MJD 2000.0)
      argv[3]    double*      radius  in km
      argv[4]    double*      theta   in degree (colatitude)
      argv[5]    double*      phi     in degree
      argv[6]    double*      est     parameter
      argv[7]    double*      ist     parameter
      argv[8]    double*      imf_by  parameter

   output:

      argv[9]    double*      internal (X,Y,Z,F)
      argv[10]   double*      external (X,Y,Z)

   returns:

      uh, something.

                 int          status, not zero, if something was wrong..

   -------------------------------------------------------------------------*/
   int      status;
   double   xm, ym, zm, fm;
   double   xe, ye, ze;
   tmodel   *model;
   double   *fday;
   double   *rad, *theta, *phi;
   double   *ist, *est, *imf_by;
   double   *bm, *be;
   long     i, len, *lptr;
   double   theta_deg, phi_deg;

   if (argc != 11)
   { 
      return(11);               /* unexpected */
   }

   status = pomme_calc_slave
      (
         (long*)   argv[0],
         (long*)   argv[1],
         (double*) argv[2],
         (double*) argv[3],
         (double*) argv[4],
         (double*) argv[5],
         (double*) argv[6],
         (double*) argv[7],
         (double*) argv[8],
         (double*) argv[9],
         (double*) argv[10]
      );

   return(status);
}

int pomme_cntl
(
   int   argc, 
   void  *argv[]
)
{
   /*-------------------------------------------------------------------------

   Master routine interfacing to calling IDL
   (with IDL portable calling convension)

   input:

      argv[0]    long         pointer to main parameter structure 'model'
      argv[1]    string       command to manipulate function

   returns:

      uh, something.

                 int          status...

   -------------------------------------------------------------------------*/
   int         status;
   long        *lmodel;
   tmodel      *model;
   char        *cmd;
   IDL_STRING  *is;

   if (argc != 2)
   { 
      printf("argc = %d\n", argc);
      return (34);              /* unexpected */
   }
   
   lmodel = (long*)   argv[0];
   model  = (tmodel*) *lmodel;

   is     = (IDL_STRING*) argv[1];
   cmd    = is->s;

   status = pomme_control(model, cmd);

   return(status);
}

/*============================================================================
   EOF pomme_idl.c
============================================================================*/
