;;=============================================================================
;;  
;;  GFZ Section 2.3 internal: interpolate combined est/ist and omni data table
;;                            assembled for internal use (leaded by MJD2000.0
;;                            time information) and read/prepare supplemental 
;;                            data needed by geo-magnetic field model POMME 2.3
;;                            code.
;;
;;  History:
;;
;;  2004-Oct-27. Martin Rother (rother@gfz-potsdam.de)
;;  2004-Dec-18. fixed typo (with impact to results)
;;=============================================================================

function istimf_calc, par, mjd 

  ;============================================================================
  ;  
  ;  returns a structure with 'interpolation'-results using structure 'par' 
  ;  with By_IMF (GSM) from selfmade data 'omni2'-all-file and the 
  ;  'est/ist'-file table contributed by stefan maus, the author of the 
  ;  'POMME 2.3' model and the 'mjd' double array with the dates/times this
  ;  values are desired.
  ;
  ;  the implicit loop should be used, because an explicit loop may be SLOW!
  ;
  ;  input:                     
  ;
  ;     varname           dim/type   comment 
  ;     -----------------------------------------------------------------------
  ;     par               struct     structure red by 'istimf_prep.pro'
  ; 
  ;     mjd               double(n)  single value or array of 'mjd.fraction'
  ;                                  values.
  ;                              
  ;  output:
  ;
  ;     this              struct     result of the interpolation and
  ;                                  tags: 
  ;                                         mjd
  ;                                         ist
  ;                                         est
  ;                                         by_imf
  ; 
  ;                                  the tags have the length of input 'mjd'.
  ;
  ;  2004-Oct-27. Martin Rother (rother@gfz-potsdam.de)
  ;  2004-Dec-18. fixed (typo) bug.
  ;===========================================================================

  ;---------------------------------------------------------------------------
  ;  USUALLY AN IMPLICIT LOOP!
  ;---------------------------------------------------------------------------

  n = n_elements(mjd)

  IF n EQ 1 THEN BEGIN 
     ;
     IF (mjd GE min(par.estist[0,*])) AND $
        (mjd LE max(par.estist[0,*])) THEN BEGIN 
        ;
        est    = interpol(par.estist[2,*], par.estist[0,*], mjd) 
        ist    = interpol(par.estist[3,*], par.estist[0,*], mjd) 
        ;
     ENDIF ELSE BEGIN 
        ;
        est    = par.avg.est
        ist    = par.avg.ist
        ;
     ENDELSE
     ;
     IF (mjd GE min(par.by_imf[0,*])) AND $
        (mjd LE max(par.by_imf[0,*])) THEN BEGIN 
        ;
        by_imf = interpol(par.by_imf[1,*], par.by_imf[0,*], mjd) ;
        ;
     ENDIF ELSE BEGIN 
        ;
        by_imf = par.avg.by_imf
        ;
     ENDELSE
     ;
  ENDIF ELSE BEGIN 
     ;
     n          = n_elements(mjd)
     mjd0       = min(mjd,max = mjd1)
     i1         = where(                                   $
                        (mjd GE min(par.estist[0,*])) AND  $
                        (mjd LE max(par.estist[0,*])),     $
                        n1,                                $
                        complement  = i1n,                 $
                        ncomplement = n1n                  $
                       )
     i2         = where(                                   $
                        (mjd GE min(par.by_imf[0,*])) AND  $
                        (mjd LE max(par.by_imf[0,*])),     $
                        n2,                                $
                        complement  = i2n,                 $
                        ncomplement = n2n                  $
                       )
     est        = replicate(par.avg.est,    n)
     ist        = replicate(par.avg.ist,    n)
     by_imf     = replicate(par.avg.by_imf, n)
     ;
     IF n1 GT 0 THEN BEGIN 
        ;
        est[i1]    = interpol(par.estist[2,*], par.estist[0,*], mjd[i1]) ;
        ist[i1]    = interpol(par.estist[3,*], par.estist[0,*], mjd[i1]) ;
        ;
     ENDIF
     ;
     IF n2 GT 0 THEN BEGIN 
        ;
        by_imf[i2] = interpol(par.by_imf[1,*], par.by_imf[0,*], mjd[i2]) ;
        ;
     ENDIF
     ;
     IF n1n GT 0 THEN $
     print, '### '+string(n1n) + ' mjd times outside estist time window'
     IF n2n GT 0 THEN $
     print, '### '+string(n2n) + ' mjd times outside by_imf time window'
     ;
  ENDELSE
  
  ;---------------------------------------------------------------------------
  ;  CREATE STRUCTURE
  ;---------------------------------------------------------------------------

  return, {                                          $
          mjd    : mjd,                              $
          est    : est,                              $
          ist    : ist,                              $
          by_imf : by_imf                            $
          }

END
;;=============================================================================
;;EOF istimf_calc.pro
;;=============================================================================
