/*  
    Magnetic field model 'Pomme' 

    Calculation

    C Core:          Stefan Maus,   October 16, 2004
    Matlab wrapper:  Martin Rother, October 17, 2004, version 2.3

    January 27, 2006: version for 'pomme-3' without significant modifications
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <pomme3d1.h>

#include <mex.h>

/*-----------------------------------------------------------------------------

   (matlab) mex wrapper

   mostly parameter checks and an outer loop for handling array-input.

   2004.Oct.16.
   2006.Jan.27. no significant modification during tests for 'pomme-3'.
-----------------------------------------------------------------------------*/

void mexFunction
(
   int nlhs,         mxArray *plhs[],
   int nrhs,   const mxArray *prhs[]     
)
{
   double  *modbuf;             /* 1 */
   double  *fday;               /* 2 */
   double  *r;                  /* 3 */
   double  *theta;              /* 4 */
   double  *phi;                /* 5 */
   double  *est;                /* 6 */
   double  *ist;                /* 7 */
   double  *imf_by;             /* 8 */

   tmodel  *model;              /* this is a structure */

   double  *bm, *be;
   double  est1, ist1, imf_by1; /* tmp var */
   double  xm, ym, zm, fm;
   double  xe, ye, ze;
   long    i;
   long    tpos_len;            /* length of time/pos arguments     */
   long    est_len;             /* length of est index parameter    */
   long    ist_len;             /* length of ist index parameter    */
   long    imf_by_len;          /* length of imf_by index parameter */

   /* 
      Examine input (right-hand-side) arguments. 
   */

   if (nrhs != 8) 
   {
      mexErrMsgTxt("Eight input parameters required.");
   }

   /* 
      check input parameter types 
   */

   if (!mxIsDouble(prhs[0]))    /* model parameter, is a scalar */
   {
      mexErrMsgTxt
         (
            "First parameter must be of type double."
         );
   }
   if (!mxIsDouble(prhs[1]))    /* time parameter, mjd */
   {
      mexErrMsgTxt
         (
            "Second parameter must be of type double."
         );
   }
   else
   {
      tpos_len = mxGetNumberOfElements(prhs[1]);
   }
   if (!mxIsDouble(prhs[2]))    /* radius position parameter */
   {
      mexErrMsgTxt
         (
            "Third parameter must be of type double."
         );
   }
   else
   {
      if (mxGetNumberOfElements(prhs[2]) != tpos_len)
      {
         mexErrMsgTxt
            (
               "Time and pos. var. must have same nr. of elements."
            );
      }
   }
   if (!mxIsDouble(prhs[3]))    /* theta position parameter */
   {
      mexErrMsgTxt
         (
            "Forth parameter must be of double type."
         );
   }
   else
   {
      if (mxGetNumberOfElements(prhs[3]) != tpos_len)
      {
         mexErrMsgTxt
            (
               "Time and pos. var. must have same nr. of elements."
            );
      }
   }
   if (!mxIsDouble(prhs[4]))    /* phi position parameter */
   {
      mexErrMsgTxt
         (
            "Fifth parameter must be of type double."
         );
   }
   else
   {
      if (mxGetNumberOfElements(prhs[4]) != tpos_len)
      {
         mexErrMsgTxt
            (
               "Time and pos. var. must have same nr. of elements."
            );
      }
   }
   if (!mxIsDouble(prhs[5]))    /* est index parameter */
   {
      mexErrMsgTxt
         (
            "Sixth parameter must be of type double."
         );
   }
   else
   {
      est_len = mxGetNumberOfElements(prhs[5]);
   }
   if (!mxIsDouble(prhs[6]))    /* ist index parameter */
   {
      mexErrMsgTxt
         (
            "Seventh parameter must be of type double."
         );
   }
   else
   {
      ist_len = mxGetNumberOfElements(prhs[6]);
   }
   if (!mxIsDouble(prhs[7]))    /* imf_by index parameter */
   {
      mexErrMsgTxt
         (
            "Eight parameter must be of type double."
         );
   }
   else
   {
      imf_by_len = mxGetNumberOfElements(prhs[7]);
   }

   if (!((est_len == 1)||(est_len == tpos_len)))
   {
      mexErrMsgTxt
         (
            "Sixth par must be scalar or of same length then time par."
         );
   }
   if (!((ist_len == 1)||(ist_len == tpos_len)))
   {
      mexErrMsgTxt
         (
            "Seventh par must be scalar or of same length then time par."
         );
   }
   if (!((imf_by_len == 1)||(imf_by_len == tpos_len)))
   {
      mexErrMsgTxt
         (
            "Eighth par must be scalar or of same length then time par."
         );
   }

   /* get 'right hand' input parameter from matlab */

   modbuf   = mxGetPr(prhs[0]); /* 1 */
   fday     = mxGetPr(prhs[1]); /* 2 */
   r        = mxGetPr(prhs[2]); /* 3 */
   theta    = mxGetPr(prhs[3]); /* 4 */
   phi      = mxGetPr(prhs[4]); /* 5 */
   est      = mxGetPr(prhs[5]); /* 6 */
   ist      = mxGetPr(prhs[6]); /* 7 */
   imf_by   = mxGetPr(prhs[7]); /* 8 */


   if (tpos_len == 1) /* -----SCALAR----- */
   {
      /* 
         create 'left hand' output parameter for matlab 
      */

      plhs[0] = mxCreateDoubleMatrix(1, 4, mxREAL); /* internal [xyzf] */
      plhs[1] = mxCreateDoubleMatrix(1, 3, mxREAL); /* external [xyz]  */

      /* 
         we need a pointer of type 'tmodel' and a temp double...
      */

      model = (tmodel*) modbuf;

      /* single call of toplevel C subroutines */

      pomme_internal_field_mp
         (
            model,
            *fday, 
            *r, *theta, *phi,               
            &xm, &ym, &zm, &fm
         );
      pomme_external_field_mp
         (
            model,
            *fday, 
            *r, *theta, *phi, 
            *est, *ist, *imf_by,
            &xe, &ye, &ze
         );

      /* copy into lefthand variables */      

      bm      = mxGetPr(plhs[0]);
      be      = mxGetPr(plhs[1]);
      
      bm[0]   = xm;   bm[1]   = ym;   bm[2]   = zm;   bm[3]   = fm;
      be[0]   = xe;   be[1]   = ye;   be[2]   = ze;

      return;
   }
   else               /* -----VECTORS----- */
   {
      /* 
         create 'left hand' output parameter for matlab 
      */

      plhs[0] = mxCreateDoubleMatrix(4,tpos_len,mxREAL); /* internal [xyzf] */
      plhs[1] = mxCreateDoubleMatrix(3,tpos_len,mxREAL); /* external [xyz]  */

      bm      = mxGetPr(plhs[0]);
      be      = mxGetPr(plhs[1]);

      /* 
         we need a pointer of type 'tmodel'
      */

      model = (tmodel*) modbuf;

      /* loop calls of toplevel C subroutines */

      for (i = 0; i < tpos_len; i++)
      {
         pomme_internal_field_mp
            (
               model,
               fday[i], 
               r[i], theta[i], phi[i],               
               &xm, &ym, &zm, &fm
            );
         if (est_len == tpos_len)
         {
            est1    = est[i];
            ist1    = ist[i];
            imf_by1 = imf_by[i];
         }
         else
         {
            est1    = *est;
            ist1    = *ist;
            imf_by1 = *imf_by;
         }
         pomme_external_field_mp
            (
               model,
               fday[i], 
               r[i], theta[i], phi[i], 
               est1, ist1, imf_by1,
               &xe, &ye, &ze
            );
         
         /* copy into lefthand variables */      

         bm[i*4    ]   = xm;   
         bm[i*4 + 1]   = ym;   
         bm[i*4 + 2]   = zm;   
         bm[i*4 + 3]   = fm;
         be[i*3    ]   = xe;   
         be[i*3 + 1]   = ye;   
         be[i*3 + 2]   = ze;
      }
   }
}

/* EOF */
