
function [v,info] = rd_n_col(mode, this, cols) 
   %---------------------------------------------------------------------------
   %
   %  simple read a file with data in n columns; 
   %  
   %  empty records or those with trailing \% will be ignored, records with a 
   %  trailing '>' are used to finish current read and return data; this is 
   %  used to handle a multi-segmented input file; do make this possible, 
   %  attempte of 'open', 'read' and 'close' are separated, for a repeated
   %  call with the 'r' option; 
   %
   %  input:                             
   %
   %     mode      string       mode flag
   %                             'o' open, this is string, filename
   %                             'r' read, this is handle
   %                             'c' close, this is handle
   %
   %  output:
   %
   %               v            depends on mode:
   % 
   %                             'o' returns handle in 'v'
   %                             'r' read: [x1,y1,z1,x2,y2,z2]  vectors
   %                             'c' returns null
   %
   %               info         structure, with counter (cnt), comments
   %                            (rem) and eof-flag (eof); if 'cnt' is zero,
   %                            no additional data is available, is [] in
   %                            open and close case.
   %
   %  example:
   %
   %        handle      = rd_n_col('o', 'datafile');
   %        [data,info] = rd_n_col('r', handle, nr_of_cols);
   %
   %        if (info.cnt == 0) 
   %           some message...
   %        endif
   %
   %        rd_n_col('c',handle);
   %
   %     if multi-segment files are expected, the 'r' call should
   %     be inside a loop, 'info.cnt' and 'info.eof' should be checked.
   %
   %  comment:
   %
   %     VERY VERY slow for big files! 
   %
   %  1k bugs, on good days some less.
   %
   %  2003-Jan-15. Martin Rother (rother@gfz-potsdam.de).
   %  2003-Apr-29. add return value in 'c' case (minor modification).
   %---------------------------------------------------------------------------

   %  CHECK AND HANDLE OPEN AND CLOSE PART

     switch mode

      case('o')
         h   = fopen(this, 'r');
         v.h = h;
         v.f = this;
         return;

      case('c')
         fclose(this.h);
         clear this;
         v = 0;
         return;

      case('r')
         n = 0;                           % a read attempt, NOP

      otherwise
         error 'illegal mode control parameter';

   end

   % TRY READ

   cnt = 0;
   rem = [];

   while (~feof(this.h));

      while (cnt == 0)                    % try an infinite read :-)

         [v, cnt] = fscanf(this.h,'%g',[cols,Inf]); 

         if (cnt == 0)

            other = fgets(this.h);
            c     = sub_str(other,1,1);

            if (c == '#')                 % comment will be stored
               other;
               rem = strvcat(rem,other);

            elseif (c == '>')             % multi-segment accepted, nop
               other;

            else                          % but nothing else
               disp(sprintf('Illegal record: %s\n',other));
               info.rem = rem;
               info.cnt = cnt;
               return;

            end

         else
                                          % oh, read was successfull
            info.fil = this.f;                                          
            info.col = cols;
            info.cnt = cnt;
            info.rem = rem;
            info.eof = feof(this.h);
            
            return;
            
         end

      end

   end
   
   info.fil = this.f;                   % EOF found, take home, whatever
   info.col = cols;
   info.cnt = cnt;
   info.rem = rem;
   info.eof = feof(this.h);
   
   %%keyboard

%%EOF rd_n_col.m   

